--------------------------------------------------------------------------------
-- Buff: Hyperspeed by Holzchopf
-- v1.1 - 07 Oct 2011	- Tweaked around the use of getturn()
--						- Fixed bug of showing info after usage
--------------------------------------------------------------------------------

-- setup
if holzchopf==nil then holzchopf={} end
holzchopf.buffhyperspeed={}
holzchopf.buffhyperspeed.buff={}
holzchopf.buffhyperspeed.gfx_icn=loadgfx("holzchopf/buff_hyperspeedicon.png")
setmidhandle(holzchopf.buffhyperspeed.gfx_icn)
holzchopf.buffhyperspeed.sfx_buff=loadsfx("holzchopf/buff.wav")

-- this hack will be removed in next version
function getplayercontrol()
	if holzchopf.buffhyperspeed.controlPlayer==playercurrent() then
		return 0
	else
		return 1
	end
end
holzchopf.buffhyperspeed.holzchopfplayercontrol=playercontrol
holzchopf.buffhyperspeed.controlPlayer=-1
function holzchopf.buffhyperspeed.playercontrol(control)
	if control==0 then
		holzchopf.buffhyperspeed.controlPlayer=playercurrent()
	else
		holzchopf.buffhyperspeed.controlPlayer=-1
	end
	holzchopf.buffhyperspeed.holzchopfplayercontrol(control)
end
playercontrol=holzchopf.buffhyperspeed.playercontrol

--------------------------------------------------------------------------------
-- Weapon
--------------------------------------------------------------------------------
holzchopf.buffhyperspeed.id=addweapon("holzchopf.buffhyperspeed","Buff: Hyperspeed",holzchopf.buffhyperspeed.gfx_icn,0,2)

function holzchopf.buffhyperspeed.draw()
	if weapon_shots==0 then
		hudinfo("Hit [attack] to activate Hyperspeed")
	end
end

function holzchopf.buffhyperspeed.attack(attack)
	if attack==1 and weapon_shots==0 then
		local player=playercurrent()
		createobject(holzchopf.buffhyperspeed.buff.id, getplayerx(player),getplayery(player),player)
		weapon_shots=1
		useweapon(0)
		endturn()
		-- fx
		particle(p_ring,getplayerx(0),getplayery(0))
		particlecolor(255,96,96)
		playsound(holzchopf.buffhyperspeed.sfx_buff)
	end
end

--------------------------------------------------------------------------------
-- Buff
--------------------------------------------------------------------------------
holzchopf.buffhyperspeed.buff.id=addobject("holzchopf.buffhyperspeed.buff",0)

function holzchopf.buffhyperspeed.buff.setup(id,player)
	-- set up buff
	objects[id]={}
	objects[id].isbuff=true
	objects[id].player=player
	objects[id].x=getplayerx(player)
	objects[id].y=getplayery(player)
	objects[id].enabled=false
end

function holzchopf.buffhyperspeed.buff.draw(id)
	local p=objects[id].player
	-- do not draw at all if player is invisible
	if getplayerstate(p,state_invisible)==1 then return 0 end
	-- draw enabled mode
	if objects[id].enabled==true then
		local x=getplayerx(p)
		local y=getplayery(p)+14
		local sx=-getplayerdirection(p)*math.random(5,20)/10
		local sy=math.random(-5,5)/10
		particle(p_flare,x,y)
		particlespeed(sx,sy)
		particlesize(2,2)
		particlecolor(255,64,64)
	-- draw disabled mode
	else
		local x=getplayerx(p)
		local y=getplayery(p)+14
		local sx=math.random(-10,10)/10
		local sy=math.random(-4,4)/10
		particle(p_flare,x,y)
		particlespeed(sx,sy)
		particlesize(2,2)
		particlecolor(64,16,16)
	end
	-- set to disabled
	objects[id].enabled=false
end

function holzchopf.buffhyperspeed.buff.update(id)
	-- reset playercontrol memorized current player
	if playercurrent()~=holzchopf.buffhyperspeed.controlPlayer then
		holzchopf.buffhyperspeed.controlPlayer=-1
	end
	local p=objects[id].player
	-- remove if player is dead
	if getplayerhealth(p)<=0 then
		removeobject(id)
		return 0
	end
	-- only update when active
	if p~=playercurrent() then return 0 end
	-- disable for frozen, confused or sleeping players
	if getplayerstate(p,state_confused)==1 or
	  getplayerstate(p,state_frozen)==1 or
	  getplayerstate(p,state_sleeping)==1 then
		return 0
	end
	-- disable for uncontrolled players (or you'll fall into water when steering RC-missile ;) )
	if getplayercontrol()==0 then return 0 end
	-- enabled
	objects[id].enabled=true
	-- get movement
	local x=getplayerx(p)
	local y=getplayery(p)
	-- move left
	if keydown(key_left)==1 then
		holzchopf.buffhyperspeed.buff.move(p,x,y,-1)
	-- move right
	elseif keydown(key_right)==1 then
		holzchopf.buffhyperspeed.buff.move(p,x,y,1)
	end
	return 0 -- (allow to) end turn
end

function holzchopf.buffhyperspeed.buff.move(player,x,y,dx)
	local stp=1
	if dx<0 then stp=-1 end
	local nx=x
	local ny=y
	local newx=x
	local newy=y
	local blocked=-1
	-- find free spot
	for sx=stp,dx,stp do
		nx=x+sx
		-- collision: walk up
		if collision(colplayer,nx,ny,1,1,0,0)==1 then
			blocked=1
			-- move up
			for sy=1,3 do
				ny=y-sy
				-- until free spot is found
				if collision(colplayer,nx,ny,1,1,0,0)==0 then
					blocked=0
					-- apply position
					newx=nx
					newy=ny
					playerposition(player,newx,newy)
					-- update ypos
					y=newy
					break
				end
			end
		-- no collision: apply position
		else
			blocked=0
			newx=nx
			newy=ny
			playerposition(player,newx,newy)
		end
		-- player blocked, don't move further
		if blocked==1 then
			break
		end
	end
end

function holzchopf.buffhyperspeed.buff.damage(id,damage)

end
