--------------------------------------------------------------------------------
-- Buff: Incredible Armour by Holzchopf
-- v1.1 - 07 Oct 2011	- Fixed bug of showing info after usage
--------------------------------------------------------------------------------

-- setup
if holzchopf==nil then holzchopf={} end
holzchopf.incrediblearmour={}
holzchopf.incrediblearmour.buff={}
holzchopf.incrediblearmour.gfx_icn=loadgfx("holzchopf/buff_incrediblearmouricon.png")
setmidhandle(holzchopf.incrediblearmour.gfx_icn)
holzchopf.incrediblearmour.gfx_buff=loadgfx("holzchopf/buff_incrediblearmour.png")
setmidhandle(holzchopf.incrediblearmour.gfx_buff)
holzchopf.incrediblearmour.sfx_buff=loadsfx("holzchopf/buff.wav")

--------------------------------------------------------------------------------
-- Weapon
--------------------------------------------------------------------------------
holzchopf.incrediblearmour.id=addweapon("holzchopf.incrediblearmour","Buff: Incredible Armour",holzchopf.incrediblearmour.gfx_icn,0,2)

function holzchopf.incrediblearmour.draw()
	if weapon_shots==0 then
		hudinfo("Hit [attack] to activate Incredible Armour")
	end
end

function holzchopf.incrediblearmour.attack(attack)
	if attack==1 and weapon_shots==0 then
		local player=playercurrent()
		createobject(holzchopf.incrediblearmour.buff.id, getplayerx(player),getplayery(player),player)
		weapon_shots=1
		useweapon(0)
		endturn()
		-- fx
		particle(p_ring,getplayerx(0),getplayery(0))
		particlecolor(64,192,255)
		playsound(holzchopf.incrediblearmour.sfx_buff)
	end
end

-- playerdamage replacement
holzchopf.incrediblearmour.holzchopfplayerdamage=playerdamage
function holzchopf.incrediblearmour.playerdamage(player,damage)
	-- damage divider
	local div=holzchopf.incrediblearmour.getdivider(player)
	-- decrease damage
	damage=math.ceil(damage/div)
	holzchopf.incrediblearmour.holzchopfplayerdamage(player,damage)
end
playerdamage=holzchopf.incrediblearmour.playerdamage
-- arealdamage replacement
holzchopf.incrediblearmour.holzchopfarealdamage=arealdamage
function holzchopf.incrediblearmour.arealdamage(x,y,radius,damage)
	-- search players in range
	local players=playertable(0,0)
	for p=1,#players do
		local x=getplayerx(p)-x
		local y=getplayery(p)-y
		local distance=math.sqrt(x*x+y*y)
		-- player in range
		if distance<radius then
			-- estimate damage
			local dam=math.floor((1-distance/radius)*damage)
			local div=holzchopf.incrediblearmour.getdivider(p)
			-- heal player if nescessary
			if div>1 then
				local realdam=math.ceil(dam/div)
				playerheal(p,dam-realdam)
			end
		end
	end
	holzchopf.incrediblearmour.holzchopfarealdamage(x,y,radius,damage)
end
arealdamage=holzchopf.incrediblearmour.arealdamage

-- return damage divider for player
function holzchopf.incrediblearmour.getdivider(p)
	-- damage divider
	local div=1
	-- check if player has incredible armour
	local ids=objecttable(holzchopf.incrediblearmour.buff.id)
	for i=1,#ids do
		local obj=objects[ids[i]]
		if obj.player~=nil then
			if obj.player==p then
				-- increase divider per armour
				div=div+1
			end
		end
	end
	return div
end

--------------------------------------------------------------------------------
-- Buff
--------------------------------------------------------------------------------
holzchopf.incrediblearmour.buff.id=addobject("holzchopf.incrediblearmour.buff",0)

function holzchopf.incrediblearmour.buff.setup(id,player)
	-- set up buff
	objects[id]={}
	objects[id].isbuff=true
	objects[id].player=player
	objects[id].x=getplayerx(player)
	objects[id].y=getplayery(player)
end

function holzchopf.incrediblearmour.buff.draw(id)
	local p=objects[id].player
	-- do not draw at all if player is invisible
	if getplayerstate(p,state_invisible)==1 then return 0 end
	local x=getplayerx(p)
	local y=getplayery(p)-2
	local s=math.sin(getframe()/10)
	setblend(blend_light)
	setalpha(1)
	setcolor(128,191,255)
	setscale(1+s*0.05,1-s*0.05)
	setrotation(0)
	drawimage(holzchopf.incrediblearmour.gfx_buff,x,y)
end

function holzchopf.incrediblearmour.buff.update(id)
	local p=objects[id].player
	-- remove if player is dead
	if getplayerhealth(p)<=0 then
		removeobject(id)
		return 0
	end
	return 0 -- (allow to) end turn
end

function holzchopf.incrediblearmour.buff.damage(id,damage)
end
