--------------------------------------------------------------------------------
-- Buff: Magic Resistance by Holzchopf
-- v1.1 - 07 Oct 2011	- Fixed immune text glitch
--						- Fixed bug of showing info after usage
--------------------------------------------------------------------------------

-- setup
if holzchopf==nil then holzchopf={} end
holzchopf.magicresistance={}
holzchopf.magicresistance.buff={}
holzchopf.magicresistance.text={}
holzchopf.magicresistance.gfx_icn=loadgfx("holzchopf/buff_magicresistanceicon.png")
setmidhandle(holzchopf.magicresistance.gfx_icn)
holzchopf.magicresistance.gfx_immune=loadgfx("holzchopf/buff_magicresistanceimmune.png")
setmidhandle(holzchopf.magicresistance.gfx_immune)
holzchopf.magicresistance.gfx_buff=loadgfx("holzchopf/buff_magicresistance.png")
setmidhandle(holzchopf.magicresistance.gfx_buff)
holzchopf.magicresistance.sfx_buff=loadsfx("holzchopf/buff.wav")

--------------------------------------------------------------------------------
-- Weapon
--------------------------------------------------------------------------------
holzchopf.magicresistance.id=addweapon("holzchopf.magicresistance","Buff: Magic Resistance",holzchopf.magicresistance.gfx_icn,1)

function holzchopf.magicresistance.draw()
	if weapon_shots==0 then
		hudinfo("Hit [attack] to activate Magic Resistance")
	end
end

function holzchopf.magicresistance.attack(attack)
	if attack==1 and weapon_shots==0 then
		local player=playercurrent()
		createobject(holzchopf.magicresistance.buff.id, getplayerx(player),getplayery(player),player)
		weapon_shots=1
		useweapon(1)
		-- fx
		particle(p_ring,getplayerx(0),getplayery(0))
		particlecolor(192,0,255)
		playsound(holzchopf.magicresistance.sfx_buff)
	end
end

-- playerstate replacement
holzchopf.magicresistance.holzchopfplayerstate=playerstate
function holzchopf.magicresistance.playerstate(player,state,enable)
	-- check if player has magic armour
	local ids=objecttable(holzchopf.magicresistance.buff.id)
	local isimmune=false
	local buffid=-1
	for i=1,#ids do
		local obj=objects[ids[i]]
		if obj.player~=nil then
			if obj.player==player then
				isimmune=true
				buffid=ids[i]
			end
		end
	end
	if isimmune==false or enable==false then
		holzchopf.magicresistance.holzchopfplayerstate(player,state,enable)
	else
		-- remove one buff
		if buffid>=0 then
			removeobject(buffid)
			local color=0xffffff
			if     state==state_poisoned then color=0x00ff00
			elseif state==state_confused then color=0xffff00
			elseif state==state_frozen then color=0xccfffff
			end
			createobject(holzchopf.magicresistance.text.id, getplayerx(player),getplayery(player)-15,color)
		end
	end
end
playerstate=holzchopf.magicresistance.playerstate

--------------------------------------------------------------------------------
-- Buff
--------------------------------------------------------------------------------
holzchopf.magicresistance.buff.id=addobject("holzchopf.magicresistance.buff",0)

function holzchopf.magicresistance.buff.setup(id,player)
	-- set up buff
	objects[id]={}
	objects[id].isbuff=true
	objects[id].player=player
	objects[id].x=getplayerx(player)
	objects[id].y=getplayery(player)
end

function holzchopf.magicresistance.buff.draw(id)
	local p=objects[id].player
	-- do not draw at all if player is invisible
	if getplayerstate(p,state_invisible)==1 then return 0 end
	local x=getplayerx(p)
	local y=getplayery(p)-2
	local t=getframe()
	setblend(blend_light)
	setalpha(1)
	setscale(1,1)
	for i=0,2 do
		local a=t*5+i*120
		local ar=math.rad(a)
		setrotation(a/3,a/3)
		local c=math.sin(2/3*math.pi*i+t/10)*128
		setcolor(128+c,0,128-c)
		drawimage(holzchopf.magicresistance.gfx_buff,x+math.cos(ar)*3,y-math.sin(ar)*3)
	end
end

function holzchopf.magicresistance.buff.update(id)
	local p=objects[id].player
	-- remove if player is dead
	if getplayerhealth(p)<=0 then
		removeobject(id)
		return 0
	end
	return 0 -- (allow to) end turn
end

function holzchopf.magicresistance.buff.damage(id,damage)
end

--------------------------------------------------------------------------------
-- Immune text
--------------------------------------------------------------------------------
holzchopf.magicresistance.text.id=addobject("holzchopf.magicresistance.text",0)

function holzchopf.magicresistance.text.setup(id,color)
	-- set up buff
	objects[id]={}
	objects[id].timer=getframe()+50
	objects[id].r=math.mod(color/0x10000,256)
	objects[id].g=math.mod(color/0x100,256)
	objects[id].b=math.mod(color,256)
end

function holzchopf.magicresistance.text.draw(id)
	setblend(blend_alpha)
	setcolor(objects[id].r,objects[id].g,objects[id].b)
	setscale(1,1)
	setrotation(0)
	drawimage(holzchopf.magicresistance.gfx_immune,getobjectx(id),getobjecty(id))
end

function holzchopf.magicresistance.text.update(id)
	if getframe()>=objects[id].timer then
		removeobject(id)
		return 0
	end
	objectposition(id,getobjectx(id),getobjecty(id)-1)
	return 1
end

function holzchopf.magicresistance.text.damage(id,damage)
end